import ntpath
import os

from flink_rest_client.common import _execute_rest_request, RestException


class JarsClient:
    def __init__(self, prefix):
        """
        Constructor.

        Parameters
        ----------
        prefix: str
            REST API url prefix. It must contain the host, port pair.
        """
        self.prefix = f"{prefix}/jars"

    def all(self):
        """
        Returns a list of all jars previously uploaded via '/jars/upload'.

        Endpoint: [GET] /jars

        Returns
        -------
        dict
            List all the jars were previously uploaded.
        """
        return _execute_rest_request(url=self.prefix)

    def upload(self, path_to_jar):
        """
        Uploads a jar to the cluster from the input path. The jar's name will be the original filename from the input
        path.

        Endpoint: [POST] /jars/upload

        Parameters
        ----------
        path_to_jar: str
            Path to the jar file.

        Returns
        -------
        dict
            Result of jar upload.
        """
        filename = os.path.basename(path_to_jar)
        files = {
            "file": (filename, (open(path_to_jar, "rb")), "application/x-java-archive")
        }
        return _execute_rest_request(
            url=f"{self.prefix}/upload", http_method="POST", files=files
        )

    def get_plan(self, jar_id):
        """
        Returns the dataflow plan of a job contained in a jar previously uploaded via '/jars/upload'.

        Endpoint: [POST] /jars/:jarid/plan

        Parameters
        ----------
        jar_id: str
            String value that identifies a jar. When uploading the jar a path is returned, where the filename is the ID.
            This value is equivalent to the `id` field in the list of uploaded jars.xe

        Returns
        -------
        dict
            Details of the jar_id's plan.

        Raises
        ------
        RestException
            If the jar_id does not exist.
        """
        return _execute_rest_request(
            url=f"{self.prefix}/{jar_id}/plan", http_method="POST"
        )["plan"]

    def run(
        self,
        jar_id,
        arguments=None,
        entry_class=None,
        parallelism=None,
        savepoint_path=None,
        allow_non_restored_state=None,
    ):
        """
        Submits a job by running a jar previously uploaded via '/jars/upload'.

        Endpoint: [POST] /jars/:jarid/run

        Parameters
        ----------
        jar_id: str
            String value that identifies a jar. When uploading the jar a path is returned, where the filename is the ID.
            This value is equivalent to the `id` field in the list of uploaded jars.

        arguments: dict
            (Optional) Dict of program arguments.

        entry_class: str
            (Optional) String value that specifies the fully qualified name of the entry point class. Overrides the
            class defined in the jar file manifest.

        parallelism: int
             (Optional) Positive integer value that specifies the desired parallelism for the job.

        savepoint_path: str
             (Optional) String value that specifies the path of the savepoint to restore the job from.

        allow_non_restored_state: bool
             (Optional) Boolean value that specifies whether the job submission should be rejected if the savepoint
             contains state that cannot be mapped back to the job.

        Returns
        -------
        str
            32-character hexadecimal string value that identifies a job.

        Raises
        ------
        RestException
            If the jar_id does not exist.
        """
        data = {}
        if arguments is not None:
            data["programArgs"] = " ".join([f"--{k} {v}" for k, v in arguments.items()])
        if entry_class is not None:
            data["entry-class"] = entry_class
        if parallelism is not None:
            if parallelism < 0:
                raise RestException(
                    "get_plan method's parallelism parameter must be a positive integer."
                )
            data["parallelism"] = parallelism
        if savepoint_path is not None:
            data["savepointPath"] = savepoint_path
        if allow_non_restored_state is not None:
            data["allowNonRestoredState"] = allow_non_restored_state

        return _execute_rest_request(
            url=f"{self.prefix}/{jar_id}/run", http_method="POST", json=data
        )["jobid"]

    def upload_and_run(
        self,
        path_to_jar,
        arguments=None,
        entry_class=None,
        parallelism=None,
        savepoint_path=None,
        allow_non_restored_state=None,
    ):
        """
        Helper method to upload and start a jar in one method call.

        Parameters
        ----------
        path_to_jar: str
            Path to the jar file.

        arguments: dict
            (Optional) Comma-separated list of program arguments.

        entry_class: str
            (Optional) String value that specifies the fully qualified name of the entry point class. Overrides the
            class defined in the jar file manifest.

        parallelism: int
             (Optional) Positive integer value that specifies the desired parallelism for the job.

        savepoint_path: str
             (Optional) String value that specifies the path of the savepoint to restore the job from.

        allow_non_restored_state: bool
             (Optional) Boolean value that specifies whether the job submission should be rejected if the savepoint
             contains state that cannot be mapped back to the job.

        Returns
        -------
        str
            32-character hexadecimal string value that identifies a job.

        Raises
        ------
        RestException
            If an error occurred during the upload of jar file.
        """
        result = self.upload(path_to_jar=path_to_jar)
        if not result["status"] == "success":
            raise RestException("Could not upload the input jar file.", result)

        return self.run(
            ntpath.basename(result["filename"]),
            arguments=arguments,
            entry_class=entry_class,
            parallelism=parallelism,
            savepoint_path=savepoint_path,
            allow_non_restored_state=allow_non_restored_state,
        )

    def delete(self, jar_id):
        """
        Deletes a jar previously uploaded via '/jars/upload'.

        Endpoint: [DELETE] /jars/:jarid

        Parameters
        ----------
        jar_id: str
            String value that identifies a jar. When uploading the jar a path is returned, where the filename is the ID.
            This value is equivalent to the `id` field in the list of uploaded jars.

        Returns
        -------
        bool
            True, if jar_id has been successfully deleted, otherwise False.

        Raises
        ------
        RestException
            If the jar_id does not exist.
        """
        res = _execute_rest_request(url=f"{self.prefix}/{jar_id}", http_method="DELETE")
        if len(res.keys()) < 1:
            return True
        else:
            return False



from flink_rest_client.common import _execute_rest_request
from flink_rest_client.v1.jars import JarsClient
from flink_rest_client.v1.jobmanager import JobmanagerClient
from flink_rest_client.v1.jobs import JobsClient
from flink_rest_client.v1.taskmanagers import TaskManagersClient


class DatasetTrigger:
    def __init__(self, prefix, trigger_id):
        self._prefix = prefix
        self.trigger_id = trigger_id

    @property
    def status(self):
        return _execute_rest_request(url=f"{self._prefix}/{self.trigger_id}")


class FlinkRestClientV1:
    def __init__(self, host, port):
        self.host = host
        self.port = port

    @property
    def api_url(self):
        return f"http://{self.host}:{str(self.port)}/v1"

    @property
    def jobmanager(self):
        return JobmanagerClient(prefix=self.api_url)

    @property
    def taskmanagers(self):
        return TaskManagersClient(prefix=self.api_url)

    @property
    def jars(self):
        return JarsClient(prefix=self.api_url)

    @property
    def jobs(self):
        return JobsClient(prefix=self.api_url)

    def overview(self):
        """
        Returns an overview over the Flink cluster.

        Endpoint: [GET] /overview

        Returns
        -------
        dict
            Key-value pairs of flink cluster infos.
        """
        return _execute_rest_request(url=f"{self.api_url}/overview")

    def config(self):
        """
        Returns the configuration of the WebUI.

        Endpoint: [GET] /config

        Returns
        -------
        dict
            Query result as a dict.
        """
        return _execute_rest_request(url=f"{self.api_url}/config", http_method="GET")

    def delete_cluster(self):
        """
        Shuts down the cluster.

        Endpoint: [DELETE] /cluster

        Returns
        -------
        dict
            Result of delete operation.
        """
        return _execute_rest_request(
            url=f"{self.api_url}/cluster", http_method="DELETE"
        )

    def datasets(self):
        """
        Returns all cluster data sets.

        Endpoint: [GET] /datasets

        Returns
        -------
        list
            Query result as a list of datasets.
        """
        return _execute_rest_request(url=f"{self.api_url}/datasets", http_method="GET")[
            "dataSets"
        ]

    def delete_dataset(self, dataset_id):
        """
        Triggers the deletion of a cluster data set. This async operation would return a DatasetTrigger for further
        query identifier.

        Endpoint: [DELETE] /datasets/:datasetid

        Parameters
        ----------
        dataset_id: str
             32-character hexadecimal string value that identifies a cluster data set.

        Returns
        -------
        DatasetTrigger
            Object that can be used to query the status of delete operation.
        """
        trigger_id = _execute_rest_request(
            url=f"{self.api_url}/datasets/{dataset_id}",
            http_method="DELETE",
            accepted_status_code=202,
        )["request-id"]
        return DatasetTrigger(
            prefix=f"{self.api_url}/datasets/delete", trigger_id=trigger_id
        )




from tests.v1.test_base import TestBase


class TestJobmanagerClient(TestBase):

    def test_config(self, simple_client, requests_mock):
        requests_mock.get(f'{simple_client.jobmanager.prefix}/config', json=[
            {
                'key': 'blob.server.port',
                'value': '6124'
            },
            {
                'key': 'taskmanager.memory.process.size',
                'value': '1728m'
            },
        ])
        response = simple_client.jobmanager.config()

        assert isinstance(response, dict)
        assert response['blob.server.port'] == '6124'
        assert response['taskmanager.memory.process.size'] == '1728m'

    def test_logs(self, simple_client, requests_mock):
        requests_mock.get(f'{simple_client.jobmanager.prefix}/logs', json={
            'logs': [{'name': 'flink--standalonesession-1-64c17dab68c5.log', 'size': 19222}]
        })
        response = simple_client.jobmanager.logs()

        assert isinstance(response, list)
        assert response[0]['name'] == 'flink--standalonesession-1-64c17dab68c5.log'

    def test_get_log(self, simple_client, requests_mock):

        log_file = "flink--standalonesession-1-64c17dab68c5.log"
        content = "content_of_the_log_file"

        requests_mock.get(f'{simple_client.jobmanager.prefix}/logs/{log_file}', text=content)
        response = simple_client.jobmanager.get_log(log_file)

        assert isinstance(response, str)
        assert response == content

    def test_metric_names(self, simple_client, requests_mock):
        requests_mock.get(f'{simple_client.jobmanager.prefix}/metrics', json=[
            {'id': 'Status.JVM.GarbageCollector.PS_MarkSweep.Time'},
            {'id': 'Status.JVM.Memory.Mapped.TotalCapacity'},

        ])
        response = simple_client.jobmanager.metric_names()

        assert isinstance(response, list)
        assert response[0] == 'Status.JVM.GarbageCollector.PS_MarkSweep.Time'
        assert response[1] == 'Status.JVM.Memory.Mapped.TotalCapacity'

    def test_metrics(self, simple_client, requests_mock):
        requests_mock.get(f'{simple_client.jobmanager.prefix}/metrics', json=[
            {'id': 'Status.JVM.GarbageCollector.PS_MarkSweep.Time',
             'value': '33'},
            {'id': 'Status.JVM.Memory.Mapped.TotalCapacity',
             'value': '0'},
        ])
        response = simple_client.jobmanager.metrics()

        assert isinstance(response, dict)
        assert response['Status.JVM.GarbageCollector.PS_MarkSweep.Time'] == '33'
        assert response['Status.JVM.Memory.Mapped.TotalCapacity'] == '0'



import requests

from flink_rest_client.common import _execute_rest_request, RestException


class JobmanagerClient:
    def __init__(self, prefix):
        """
        Constructor.

        Parameters
        ----------
        prefix: str
            REST API url prefix. It must contain the host, port pair.
        """
        self.prefix = f"{prefix}/jobmanager"

    def config(self):
        """
        Returns the cluster configuration.

        Endpoint: [GET] /jobmanager/config

        Returns
        -------
        dict
            Cluster configuration dictionary.
        """
        query_result = _execute_rest_request(url=f"{self.prefix}/config")
        return dict([(elem["key"], elem["value"]) for elem in query_result])

    def logs(self):
        """
        Returns the list of log files on the JobManager.

        Endpoint: [GET] /jobmanager/logs

        Returns
        -------
        dict
            List of log files
        """
        return _execute_rest_request(url=f"{self.prefix}/logs")["logs"]

    def get_log(self, log_file):
        """
        Returns the content of the log_file.

        Endpoint: [GET] /jobmanager/logs/:log_file

        Parameters
        ----------
        log_file: str
            Name of the log file.
        Returns
        -------
        str
            The content of the log file as a string
        """
        response = requests.request(method="GET", url=f"{self.prefix}/logs/{log_file}")
        if response.status_code == 200:
            return response.content.decode()
        else:
            if "errors" in response.json().keys():
                error_str = "\n".join(response.json()["errors"])
            else:
                error_str = ""
            raise RestException(
                f"REST response error ({response.status_code}): {error_str}"
            )

    def metric_names(self):
        """
        Return the supported metric names.

        Returns
        -------
        list
            List of metric names.
        """
        return [
            elem["id"] for elem in _execute_rest_request(url=f"{self.prefix}/metrics")
        ]

    def metrics(self):
        """
        Provides access to job manager metrics.

         Endpoint: [GET] /jobmanager/metrics

         Returns
         -------
         dict
             Jobmanager metrics
        """
        metric_names = self.metric_names()
        params = {"get": ",".join(metric_names)}
        query_result = _execute_rest_request(
            url=f"{self.prefix}/metrics", params=params
        )
        return dict([(elem["id"], elem["value"]) for elem in query_result])



import math

from flink_rest_client.v1.jobs import JobTrigger, JobVertexClient
from tests.v1.test_base import TestBase


class TestJobsClient(TestBase):

    def test_all(self, simple_client, requests_mock):
        requests_mock.get(f'{simple_client.jobs.prefix}', json={'jobs': [
            {'id': 'a0d4b5b51065202b788bbd0a80251a3c', 'status': 'RUNNING'}
        ]})
        response = simple_client.jobs.all()

        assert isinstance(response, list)
        assert len(response) == 1
        assert response[0]['id'] == 'a0d4b5b51065202b788bbd0a80251a3c'

    def test_job_ids(self, simple_client, requests_mock):
        requests_mock.get(f'{simple_client.jobs.prefix}', json={'jobs': [
            {'id': 'a0d4b5b51065202b788bbd0a80251a3c', 'status': 'RUNNING'}
        ]})
        response = simple_client.jobs.job_ids()

        assert isinstance(response, list)
        assert len(response) == 1
        assert response[0] == 'a0d4b5b51065202b788bbd0a80251a3c'

    def test_overview(self, simple_client, requests_mock):
        requests_mock.get(f'{simple_client.jobs.prefix}/overview', json={'jobs': [
            {
                'jid': 'a0d4b5b51065202b788bbd0a80251a3c',
                'name': 'State machine job',
                'state': 'RUNNING'
            }
        ]})
        response = simple_client.jobs.overview()

        assert isinstance(response, list)
        assert len(response) == 1
        assert response[0]['jid'] == 'a0d4b5b51065202b788bbd0a80251a3c'
        assert response[0]['name'] == 'State machine job'
        assert response[0]['state'] == 'RUNNING'

    def test_metric_names(self, simple_client, requests_mock):
        requests_mock.get(f'{simple_client.jobs.prefix}/metrics', json=[
            {'id': 'numberOfFailedCheckpoints'},
            {'id': 'lastCheckpointSize'},
            {'id': 'lastCheckpointExternalPath'},
        ])
        response = simple_client.jobs.metric_names()

        assert isinstance(response, list)
        assert response[0] == 'numberOfFailedCheckpoints'
        assert response[1] == 'lastCheckpointSize'
        assert response[2] == 'lastCheckpointExternalPath'

    def test_metrics(self, simple_client, requests_mock):
        requests_mock.get(f'{simple_client.jobs.prefix}', json={'jobs': [
            {'id': 'a0d4b5b51065202b788bbd0a80251a3c', 'status': 'RUNNING'}
        ]})

        requests_mock.get(f'{simple_client.jobs.prefix}/metrics', json=[
            {'id': 'numberOfFailedCheckpoints',
             'min': 0.0, 'max': 0.0, 'avg': 0.0, 'sum': 0.0},
            {'id': 'lastCheckpointSize',
             'min': 8482.0, 'max': 8482.0, 'avg': 8482.0, 'sum': 8482.0},
        ])
        response = simple_client.jobs.metrics()

        assert isinstance(response, dict)
        assert math.isclose(response['lastCheckpointSize']['min'], 8482.0)
        assert math.isclose(response['numberOfFailedCheckpoints']['min'], 0.0)

    def test_get(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}', json={
            'jid': 'a0d4b5b51065202b788bbd0a80251a3c',
            'name': 'State machine job',
            'isStoppable': False
        })
        response = simple_client.jobs.get(jid)

        assert isinstance(response, dict)
        assert response['jid'] == jid
        assert response['name'] == 'State machine job'
        assert response['isStoppable'] is False

    def test_config(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}/config', json={
            'jid': 'a0d4b5b51065202b788bbd0a80251a3c',
            'name': 'State machine job',
            'execution-config': {
                'execution-mode': 'PIPELINED',
                'restart-strategy': 'Cluster level default restart strategy',
                'job-parallelism': 1,
                'object-reuse-mode': False,
                'user-config': {}
            }
        })
        response = simple_client.jobs.get_config(jid)

        assert isinstance(response, dict)
        assert response['jid'] == jid
        assert response['name'] == 'State machine job'
        assert isinstance(response['execution-config'], dict)

    def test_exceptions(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}/exceptions', json={
            'root-exception': None, 'timestamp': None,
            'all-exceptions': []
        })
        response = simple_client.jobs.get_exceptions(jid)

        assert isinstance(response, dict)
        assert response['root-exception'] is None
        assert response['timestamp'] is None
        assert isinstance(response['all-exceptions'], list)

    def test_execution_result(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}/execution-result', json={
            'status': {'id': 'IN_PROGRESS'}
        })
        response = simple_client.jobs.get_execution_result(jid)

        assert isinstance(response, dict)
        assert response['status']['id'] == 'IN_PROGRESS'

    def test_get_metrics(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'

        requests_mock.get(f'{simple_client.jobs.prefix}/metrics', json=[
            {'id': 'numberOfFailedCheckpoints'},
            {'id': 'lastCheckpointSize'},
        ])

        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}/metrics', json=[
            {'id': 'numberOfFailedCheckpoints',
             'value': 0.0},
            {'id': 'lastCheckpointSize',
             'value': 8500.0},
        ])
        response = simple_client.jobs.get_metrics(jid)

        assert isinstance(response, dict)
        assert math.isclose(response['lastCheckpointSize'], 8500.0)
        assert math.isclose(response['numberOfFailedCheckpoints'], 0.0)

    def test_get_plan(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}/plan', json={
            'plan': {
                'jid': 'a0d4b5b51065202b788bbd0a80251a3c',
                'name': 'State machine job',
                'nodes': []
            }
        })
        response = simple_client.jobs.get_plan(jid)

        assert isinstance(response, dict)
        assert response['jid'] == 'a0d4b5b51065202b788bbd0a80251a3c'
        assert response['name'] == 'State machine job'
        assert isinstance(response['nodes'], list)

    def test_get_vertex_ids(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}', json={
            'jid': 'a0d4b5b51065202b788bbd0a80251a3c',
            'name': 'State machine job',
            'isStoppable': False,
            'vertices': [
                {'id': 'bc764cd8ddf7a0cff126f51c16239658',
                 'name': 'Source: Custom Source', 'parallelism': 1},
                {'id': '20ba6b65f97481d5570070de90e4e791',
                 'name': 'Flat Map -> Sink: Print to Std. Out', 'parallelism': 1}]

        })
        response = simple_client.jobs.get_vertex_ids(jid)

        assert isinstance(response, list)
        assert response[0] == 'bc764cd8ddf7a0cff126f51c16239658'
        assert response[1] == '20ba6b65f97481d5570070de90e4e791'

    def test_get_accumulators(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}/accumulators', json={
            'job-accumulators': [],
            'user-task-accumulators': [],
            'serialized-user-task-accumulators': {}})
        response = simple_client.jobs.get_accumulators(jid)

        assert isinstance(response, dict)
        assert isinstance(response['job-accumulators'], list)
        assert isinstance(response['user-task-accumulators'], list)
        assert isinstance(response['serialized-user-task-accumulators'], dict)

    def test_get_checkpointing_configuration(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}/checkpoints/config', json={
            'mode': 'exactly_once',
            'interval': 2000,
            'timeout': 600000})
        response = simple_client.jobs.get_checkpointing_configuration(jid)

        assert isinstance(response, dict)
        assert response['mode'] == 'exactly_once'
        assert response['interval'] == 2000
        assert response['timeout'] == 600000

    def test_get_checkpoints(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}/checkpoints', json={
            'counts': {'restored': 0, 'total': 22645},
            'summary': {}})
        response = simple_client.jobs.get_checkpoints(jid)

        assert isinstance(response, dict)
        assert isinstance(response['counts'], dict)
        assert isinstance(response['summary'], dict)
        assert response['counts']['restored'] == 0

    def test_get_checkpoint_ids(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}/checkpoints', json={
            'counts': {'restored': 0, 'total': 22645},
            'summary': {},
            'history': [
                {'@class': 'completed', 'id': 22819, 'status': 'COMPLETED'},
                {'@class': 'completed', 'id': 22818, 'status': 'COMPLETED'},
                {'@class': 'completed', 'id': 22817, 'status': 'COMPLETED'},
                {'@class': 'completed', 'id': 22816, 'status': 'COMPLETED'}
            ]
        })
        response = simple_client.jobs.get_checkpoint_ids(jid)

        assert isinstance(response, list)
        assert len(response) == 4
        assert response[0] == 22819

    def test_get_checkpoint_details(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        cid = 23342
        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}/checkpoints/details/{cid}', json={
            '@class': 'completed',
            'id': cid,
            'status': 'COMPLETED',
            'is_savepoint': False,
            'state_size': 8356,
            'end_to_end_duration': 2
        })
        response = simple_client.jobs.get_checkpoint_details(jid, cid)

        assert isinstance(response, dict)
        assert response['id'] == cid

    def test_get_checkpoint_details_with_subtasks(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        cid = 23342
        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}/checkpoints/details/{cid}', json={
            '@class': 'completed',
            'id': cid,
            'status': 'COMPLETED',
            'is_savepoint': False,
            'state_size': 8356,
            'end_to_end_duration': 2,
            'tasks': {
                '20ba6b65f97481d5570070de90e4e791': {'id': 23522, 'status': 'COMPLETED'}
            }
        })

        requests_mock.get(
            f'{simple_client.jobs.prefix}/{jid}/checkpoints/details/{cid}/subtasks/20ba6b65f97481d5570070de90e4e791',
            json={
                '@class': 'completed',
                'id': cid,
                'status': 'COMPLETED',
                'is_savepoint': False,
                'state_size': 8356,
                'end_to_end_duration': 2,
                'tasks': {
                    '20ba6b65f97481d5570070de90e4e791': {'id': 23522, 'status': 'COMPLETED'},
                    'bc764cd8ddf7a0cff126f51c16239658': {'id': 23522, 'status': 'COMPLETED'}
                }
            })

        response = simple_client.jobs.get_checkpoint_details(jid, cid, show_subtasks=True)

        assert isinstance(response, dict)
        assert isinstance(response['subtasks'], dict)
        assert response['id'] == cid
        assert len(response['subtasks']) == 1

    def test_rescale(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        tid = 'test_trigger_id'
        parallelism = 2

        requests_mock.patch(f'{simple_client.jobs.prefix}/{jid}/rescaling', json={
            'triggerid': tid,
        })

        response = simple_client.jobs.rescale(jid, parallelism)

        assert isinstance(response, JobTrigger)
        assert response.trigger_id == tid

    def test_savepoints(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        tid = 'test_trigger_id'
        path = 'test'

        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}/savepoints/{tid}', json={
            'status': {'id': 'COMPLETED'},
            'operation': {'location': 'file:/opt/flink/test/savepoint-0754b5-9e6dc83c00ac'}
        })
        requests_mock.post(f'{simple_client.jobs.prefix}/{jid}/savepoints', json={
            'request-id': tid,
        }, status_code=202)

        response = simple_client.jobs.create_savepoint(jid, path)

        assert isinstance(response, JobTrigger)
        assert response.trigger_id == tid

        assert response.status['status']['id'] == 'COMPLETED'

    def test_terminate(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        requests_mock.patch(f'{simple_client.jobs.prefix}/{jid}', json={}, status_code=202)
        response = simple_client.jobs.terminate(jid)
        assert response is True

    def test_stop(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        tid = 'test_trigger_id'
        path = 'test'

        requests_mock.get(f'{simple_client.jobs.prefix}/{jid}/savepoints/{tid}', json={
            'status': {'id': 'COMPLETED'},
            'operation': {'location': 'file:/opt/flink/test/savepoint-0754b5-9e6dc83c00ac'}
        })
        requests_mock.post(f'{simple_client.jobs.prefix}/{jid}/stop', json={
            'request-id': tid,
        }, status_code=202)

        response = simple_client.jobs.stop(jid, path)

        assert isinstance(response, JobTrigger)
        assert response.trigger_id == tid

        assert response.status['status']['id'] == 'COMPLETED'

    def test_get_vertex(self, simple_client, requests_mock):
        jid = 'a0d4b5b51065202b788bbd0a80251a3c'
        vertex_id = 'test_vertex_id'

        response = simple_client.jobs.get_vertex(jid, vertex_id)

        assert isinstance(response, JobVertexClient)
        assert response.job_id == jid
        assert response.vertex_id == vertex_id



from flink_rest_client.common import _execute_rest_request, RestException


class TaskManagersClient:
    def __init__(self, prefix):
        """
        Constructor.

        Parameters
        ----------
        prefix: str
            REST API url prefix. It must contain the host, port pair.
        """
        self.prefix = f"{prefix}/taskmanagers"

    def all(self):
        """
        Returns an overview over all task managers.

        Endpoint: [GET] /taskmanagers

        Returns
        -------
        list
            List of taskmanagers. Each taskmanager is represented by a dictionary.
        """
        return _execute_rest_request(url=self.prefix)["taskmanagers"]

    def taskmanager_ids(self):
        """
        Returns the list of taskmanager_ids.

        Returns
        -------
        list
            List of taskmanager ids.
        """
        return [elem["id"] for elem in self.all()]

    def metric_names(self):
        """
        Return the supported metric names.

        Returns
        -------
        list
            List of metric names.
        """
        return [
            elem["id"] for elem in _execute_rest_request(url=f"{self.prefix}/metrics")
        ]

    def metrics(self, metric_names=None, agg_modes=None, taskmanager_ids=None):
        """
        Provides access to aggregated task manager metrics.
        By default it returns with all existing metric names.

        Endpoint: [GET] /taskmanagers/metrics

        Parameters
        ----------
        metric_names: list
            (optional) List of selected specific metric names. Default: <all metrics>

        agg_modes: list
            (optional) List of aggregation modes which should be calculated. Available aggregations are: "min, max,
            sum, avg". Default: <all modes>

        taskmanager_ids: list
            List of 32-character hexadecimal strings to select specific task managers. The list of valid taskmanager ids
            are available through the taskmanager_ids() method. Default: <all taskmanagers>.

        Returns
        -------
        dict
            Key-value pairs of metrics.
        """

        if metric_names is None:
            metric_names = self.metric_names()

        supported_agg_modes = ["min", "max", "sum", "avg"]
        if agg_modes is None:
            agg_modes = supported_agg_modes
        if len(set(agg_modes).difference(set(supported_agg_modes))) > 0:
            raise RestException(
                f"The provided aggregation modes list contains invalid value. Supported aggregation "
                f"modes: {','.join(supported_agg_modes)}; given list: {','.join(agg_modes)}"
            )

        if taskmanager_ids is None:
            taskmanager_ids = self.taskmanager_ids()

        params = {
            "get": ",".join(metric_names),
            "agg": ",".join(agg_modes),
            "taskmanagers": ",".join(taskmanager_ids),
        }
        query_result = _execute_rest_request(
            url=f"{self.prefix}/metrics", params=params
        )

        result = {}
        for elem in query_result:
            metric_name = elem.pop("id")
            result[metric_name] = elem

        return result

    def get(self, taskmanager_id):
        """
        Returns details for a task manager.

        Endpoint: [GET] /taskmanagers/:taskmanagerid

        Parameters
        ----------
        taskmanager_id: str
            32-character hexadecimal string that identifies a task manager.

        Returns
        -------
        dict
            Query result as a dict.
        """
        return _execute_rest_request(url=f"{self.prefix}/{taskmanager_id}")

    def get_logs(self, taskmanager_id):
        """
        Returns the list of log files on a TaskManager.

        Endpoint: [GET] /taskmanagers/:taskmanagerid/logs

        Parameters
        ----------
        taskmanager_id: str
            32-character hexadecimal string that identifies a task manager.

        Returns
        -------
        list
            List of log files in which each element contains a name and size fields.
        """
        return _execute_rest_request(url=f"{self.prefix}/{taskmanager_id}/logs")["logs"]

    def get_metrics(self, taskmanager_id, metric_names=None):
        """
        Provides access to task manager metrics.

        Endpoint: [GET] /taskmanagers/:taskmanagerid/metrics

        Parameters
        ----------
        taskmanager_id: str
            32-character hexadecimal string that identifies a task manager.

        metric_names: list
            (optional) List of selected specific metric names. Default: <all metrics>

        Returns
        -------
        dict
            Metric name -> Metric value key-value pairs. The values are provided as strings.
        """

        if metric_names is None:
            metric_names = self.metric_names()
        params = {"get": ",".join(metric_names)}

        query_result = _execute_rest_request(
            url=f"{self.prefix}/{taskmanager_id}/metrics", params=params
        )
        return dict([(elem["id"], elem["value"]) for elem in query_result])

    def get_thread_dump(self, taskmanager_id):
        """
        Returns the thread dump of the requested TaskManager.

        Endpoint: [GET] /taskmanagers/:taskmanagerid/thread-dump

        Parameters
        ----------
        taskmanager_id: str
            32-character hexadecimal string that identifies a task manager.

        Returns
        -------
        dict
            ThreadName -> StringifiedThreadInfo key-value pairs.
        """
        query_result = _execute_rest_request(
            url=f"{self.prefix}/{taskmanager_id}/thread-dump"
        )["threadInfos"]
        return dict(
            [
                (elem["threadName"], elem["stringifiedThreadInfo"])
                for elem in query_result
            ]
        )



from setuptools import setup, find_packages


def readme():
    with open('README.md') as f:
        return f.read()


setup(
    name='flink-rest-client',
    version='1.0.6',
    description='Easy to use Flink REST API client implementation',
    long_description=readme(),
    long_description_content_type='text/markdown',
    url='https://github.com/frego-dev/flink-rest-client',
    author='Attila Nagy',
    author_email='anagy@frego.dev',
    license='MIT',
    packages=find_packages(),
    install_requires=['requests', 'importlib_resources'],
    entry_points={
          'console_scripts': [],
    }
)



import math

from tests.v1.test_base import TestBase


class TestTaskmanagerClient(TestBase):

    def test_all(self, simple_client, requests_mock):
        requests_mock.get(f'{simple_client.taskmanagers.prefix}', json={'taskmanagers': [
            {
                'id': '172.18.0.3:42073-c8a6ca',
                'path': 'akka.tcp://flink@172.18.0.3:42073/user/rpc/taskmanager_0',
                'dataPort': 45075
            }
        ]})
        response = simple_client.taskmanagers.all()

        assert isinstance(response, list)
        assert len(response) == 1
        assert response[0]['id'] == '172.18.0.3:42073-c8a6ca'

    def test_taskmanager_ids(self, simple_client, requests_mock):
        requests_mock.get(f'{simple_client.taskmanagers.prefix}', json={'taskmanagers': [
            {
                'id': '172.18.0.3:42073-c8a6ca',
                'path': 'akka.tcp://flink@172.18.0.3:42073/user/rpc/taskmanager_0',
                'dataPort': 45075
            }
        ]})
        response = simple_client.taskmanagers.taskmanager_ids()

        assert isinstance(response, list)
        assert len(response) == 1
        assert response[0] == '172.18.0.3:42073-c8a6ca'

    def test_metric_names(self, simple_client, requests_mock):
        requests_mock.get(f'{simple_client.taskmanagers.prefix}/metrics', json=[
            {'id': 'Status.JVM.GarbageCollector.PS_MarkSweep.Time'},
            {'id': 'Status.JVM.Memory.Mapped.TotalCapacity'},

        ])
        response = simple_client.taskmanagers.metric_names()

        assert isinstance(response, list)
        assert response[0] == 'Status.JVM.GarbageCollector.PS_MarkSweep.Time'
        assert response[1] == 'Status.JVM.Memory.Mapped.TotalCapacity'

    def test_metrics(self, simple_client, requests_mock):
        requests_mock.get(f'{simple_client.taskmanagers.prefix}', json={'taskmanagers': [
            {
                'id': '172.18.0.3:42073-c8a6ca',
                'path': 'akka.tcp://flink@172.18.0.3:42073/user/rpc/taskmanager_0',
                'dataPort': 45075
            }
        ]})

        requests_mock.get(f'{simple_client.taskmanagers.prefix}/metrics', json=[
            {'id': 'Status.Network.AvailableMemorySegments',
             'min': 4092.0, 'max': 4092.0, 'avg': 4092.0, 'sum': 4092.0},
            {'id': 'Status.JVM.Memory.Mapped.TotalCapacity',
             'min': 0.0, 'max': 0.0, 'avg': 0.0, 'sum': 0.0},
        ])
        response = simple_client.taskmanagers.metrics()

        assert isinstance(response, dict)
        assert math.isclose(response['Status.Network.AvailableMemorySegments']['min'], 4092.0)
        assert math.isclose(response['Status.JVM.Memory.Mapped.TotalCapacity']['min'], 0.0)

    def test_get(self, simple_client, requests_mock):
        tid = '172.18.0.3:42073-c8a6ca'
        requests_mock.get(f'{simple_client.taskmanagers.prefix}/{tid}', json={
            'id': '172.18.0.3:42073-c8a6ca',
            'path': 'akka.tcp://flink@172.18.0.3:42073/user/rpc/taskmanager_0',
            'dataPort': 45075,
        })
        response = simple_client.taskmanagers.get(tid)

        assert isinstance(response, dict)
        assert response['id'] == '172.18.0.3:42073-c8a6ca'
        assert response['path'] == 'akka.tcp://flink@172.18.0.3:42073/user/rpc/taskmanager_0'

    def test_get_logs(self, simple_client, requests_mock):
        tid = '172.18.0.3:42073-c8a6ca'
        requests_mock.get(f'{simple_client.taskmanagers.prefix}/{tid}/logs', json={
            'logs': [
                {'name': 'flink--taskexecutor-0-88d2501520f8.log', 'size': 18965}]

        })
        response = simple_client.taskmanagers.get_logs(tid)

        assert isinstance(response, list)
        assert len(response) == 1
        assert response[0]['name'] == 'flink--taskexecutor-0-88d2501520f8.log'

    def test_get_metrics(self, simple_client, requests_mock):
        tid = '172.18.0.3:42073-c8a6ca'

        requests_mock.get(f'{simple_client.taskmanagers.prefix}/metrics', json=[
            {'id': 'Status.JVM.GarbageCollector.PS_MarkSweep.Time'},
            {'id': 'Status.JVM.Memory.Mapped.TotalCapacity'},

        ])

        requests_mock.get(f'{simple_client.taskmanagers.prefix}/{tid}/metrics', json=[
            {
                'id': 'Status.Network.AvailableMemorySegments',
                'value': '4092'
            },
            {
                'id': 'Status.JVM.Memory.Mapped.TotalCapacity',
                'value': '0'
            },
        ])
        response = simple_client.taskmanagers.get_metrics(tid)

        assert isinstance(response, dict)
        assert len(response) == 2
        assert response['Status.Network.AvailableMemorySegments'] == '4092'

    def test_get_thread_dump(self, simple_client, requests_mock):
        tid = '172.18.0.3:42073-c8a6ca'
        requests_mock.get(f'{simple_client.taskmanagers.prefix}/{tid}/thread-dump', json={
            'threadInfos': [{
                'threadName': 'flink-taskexecutor-io-thread-1',
                'stringifiedThreadInfo': '"flink-taskexecutor-io-thread-1"'
            }]

        })
        response = simple_client.taskmanagers.get_thread_dump(tid)

        assert isinstance(response, dict)
        assert len(response) == 1
        assert response['flink-taskexecutor-io-thread-1'] == '"flink-taskexecutor-io-thread-1"'
